@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[2]:
+# In[1]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
         pass
 
 
-# In[3]:
+# In[94]:
 
 
-version = '0.51'
+version = '0.52'
 
 import re
 import tempfile
@@ -32,7 +32,7 @@ import stat
 
 # # Helper Functions
 
-# In[4]:
+# In[95]:
 
 
 def is_testing():
@@ -42,7 +42,7 @@ def is_testing():
         return False
 
 
-# In[5]:
+# In[96]:
 
 
 def isnotebook():
@@ -53,7 +53,7 @@ def isnotebook():
     return False
 
 
-# In[6]:
+# In[97]:
 
 
 def testPrint(*args):
@@ -63,7 +63,7 @@ def testPrint(*args):
 
 # # Requiremtns
 
-# In[7]:
+# In[98]:
 
 
 requirements = []
@@ -92,14 +92,14 @@ from IPython.core.display import display, HTML
 
 # # Lists
 
-# In[8]:
+# In[99]:
 
 
 def listsGetPairsFromTwoLists(list1, list2):
     return list(zip(list1, list2))
 
 
-# In[9]:
+# In[100]:
 
 
 if is_testing():
@@ -109,14 +109,14 @@ if is_testing():
     print(listsGetPairsFromTwoLists(listAbcd, list1234))
 
 
-# In[10]:
+# In[101]:
 
 
 def listSequentialOverlappingPairs(l):
     return list(zip(l[:-1], l[1:]))
 
 
-# In[11]:
+# In[102]:
 
 
 if is_testing():
@@ -132,7 +132,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[12]:
+# In[103]:
 
 
 regex_cache = {}
@@ -142,35 +142,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[13]:
+# In[104]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[14]:
+# In[105]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[15]:
+# In[106]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[16]:
+# In[107]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[17]:
+# In[108]:
 
 
 def lCsv(l, force_oneline=False):
@@ -186,21 +186,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[18]:
+# In[109]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[19]:
+# In[110]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[20]:
+# In[111]:
 
 
 if is_testing():
@@ -209,26 +209,28 @@ if is_testing():
 
 # # Text files
 
-# In[21]:
+# In[184]:
 
 
 def txtread(path):
     if 'http://' in path or 'https://' in path:
         return urlopen(path).read().decode()
+    elif 'gs://' in path:
+        gsTxtread(path)
     else:
         with open(path, 'r') as f:
             return f.read()
 
 def txtwrite(path, txt):
     if 'gs://' in path:
-        gsTxtread(path, txt)
+        gsTxtwrite(path, txt)
     else:
         os.makedirs(os.path.dirname(path), exist_ok=True)
         with open(path, 'w') as f:
             return f.write(txt)
 
 
-# In[22]:
+# In[113]:
 
 
 if is_testing():
@@ -237,7 +239,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[23]:
+# In[114]:
 
 
 regex_cache = {}
@@ -249,35 +251,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[24]:
+# In[115]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[25]:
+# In[116]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[26]:
+# In[117]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[27]:
+# In[118]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[28]:
+# In[119]:
 
 
 if is_testing():
@@ -286,7 +288,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[29]:
+# In[120]:
 
 
 import warnings
@@ -310,7 +312,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[30]:
+# In[121]:
 
 
 def gsGsToHttp(path):
@@ -320,7 +322,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[31]:
+# In[122]:
 
 
 def gsLs(path):
@@ -330,7 +332,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[32]:
+# In[123]:
 
 
 def gsUploadFile(gsPath, path):
@@ -338,7 +340,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[33]:
+# In[124]:
 
 
 def gsTxtread(path):
@@ -350,7 +352,15 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[34]:
+# In[182]:
+
+
+def gsTxtwrite(path, s):
+    blob = _gsBlob(path)
+    return blob.upload_from_string(s)
+
+
+# In[125]:
 
 
 def gsImwrite(path, img):
@@ -359,7 +369,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[35]:
+# In[126]:
 
 
 def gsImread(path):
@@ -370,7 +380,7 @@ def gsImread(path):
 
 # # Images
 
-# In[36]:
+# In[127]:
 
 
 def imread(path):
@@ -404,7 +414,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[37]:
+# In[128]:
 
 
 def imshow(array, scale=1):
@@ -419,7 +429,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[38]:
+# In[129]:
 
 
 if is_testing():
@@ -427,14 +437,14 @@ if is_testing():
     imshow(img)
 
 
-# In[39]:
+# In[130]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[40]:
+# In[131]:
 
 
 def imscaleNN(img, s):
@@ -444,7 +454,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[41]:
+# In[132]:
 
 
 if is_testing():
@@ -463,7 +473,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[42]:
+# In[133]:
 
 
 def imRepeat(img, height, width):
@@ -477,7 +487,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[43]:
+# In[134]:
 
 
 if is_testing():
@@ -485,33 +495,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[44]:
+# In[135]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[45]:
+# In[136]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[46]:
+# In[137]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[47]:
+# In[138]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[48]:
+# In[139]:
 
 
 def imAddNoiseGauss(img, std):
@@ -523,7 +533,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[49]:
+# In[140]:
 
 
 def imJpgDegradation(img, quality):
@@ -536,7 +546,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[50]:
+# In[141]:
 
 
 def imCropCenter(img, size):
@@ -551,35 +561,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[51]:
+# In[142]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[52]:
+# In[143]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[53]:
+# In[144]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[54]:
+# In[145]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[55]:
+# In[146]:
 
 
 def imGallery(imgs, pad=0):
@@ -610,7 +620,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[56]:
+# In[147]:
 
 
 if is_testing():
@@ -618,49 +628,49 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[57]:
+# In[148]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[58]:
+# In[149]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[59]:
+# In[150]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[60]:
+# In[151]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[61]:
+# In[152]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[62]:
+# In[153]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[63]:
+# In[154]:
 
 
 if is_testing():
@@ -670,14 +680,14 @@ if is_testing():
 
 # # Files
 
-# In[64]:
+# In[155]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[65]:
+# In[156]:
 
 
 if is_testing():
@@ -706,7 +716,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[66]:
+# In[157]:
 
 
 def fiRemove(path):
@@ -734,7 +744,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[67]:
+# In[158]:
 
 
 if is_testing():
@@ -768,7 +778,7 @@ if is_testing():
 
 # # Jupyter
 
-# In[68]:
+# In[159]:
 
 
 def jnNbFullWidth():
@@ -782,7 +792,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', None)
 
 
-# In[69]:
+# In[160]:
 
 
 def jnImageRenderingPixelated():
@@ -794,7 +804,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[70]:
+# In[161]:
 
 
 def jnDefault():
@@ -806,7 +816,7 @@ if is_testing():
     jnDefault()
 
 
-# In[71]:
+# In[162]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -819,7 +829,7 @@ if is_testing():
     
 
 
-# In[72]:
+# In[163]:
 
 
 def jnImgToHtmlB64(img):
@@ -843,7 +853,7 @@ def jnImageRow(imgs, names=None, width=100):
     display(HTML(html))
 
 
-# In[73]:
+# In[164]:
 
 
 if is_testing():
@@ -851,7 +861,7 @@ if is_testing():
    
 
 
-# In[74]:
+# In[165]:
 
 
 
@@ -861,7 +871,7 @@ if is_testing():
 
 # # Shell
 
-# In[75]:
+# In[166]:
 
 
 def shell(cmd):
@@ -876,7 +886,7 @@ def shell(cmd):
 
 # # GIT
 
-# In[76]:
+# In[167]:
 
 
 def gitGetHash():
@@ -886,7 +896,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[77]:
+# In[168]:
 
 
 def gitGetBranch():
@@ -896,7 +906,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[78]:
+# In[169]:
 
 
 def gitGetMessage():
@@ -906,7 +916,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[79]:
+# In[170]:
 
 
 def gitGetAllFiles():
@@ -916,7 +926,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[80]:
+# In[171]:
 
 
 def gitListByTime(path=None):
@@ -932,7 +942,7 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[81]:
+# In[172]:
 
 
 def shellSortDirsBySize():
@@ -944,14 +954,14 @@ if is_testing():
 
 # # Package Files
 
-# In[82]:
+# In[173]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[83]:
+# In[174]:
 
 
 def shell(cmd):
@@ -959,7 +969,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[84]:
+# In[175]:
 
 
 if is_testing():
@@ -991,7 +1001,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[85]:
+# In[176]:
 
 
 if is_testing():
@@ -1001,7 +1011,7 @@ description-file = README.md
 """
 
 
-# In[86]:
+# In[177]:
 
 
 if is_testing():
@@ -1024,14 +1034,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[87]:
+# In[178]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[88]:
+# In[179]:
 
 
 if is_testing():
@@ -1040,7 +1050,7 @@ if is_testing():
     
 
 
-# In[89]:
+# In[180]:
 
 
 if is_testing():
@@ -1051,7 +1061,7 @@ lx/__init__.py
 """
 
 
-# In[91]:
+# In[181]:
 
 
 if is_testing():
